--   Programming Language Technology:
--   Putting Formal Languages to Work
--
--   Andreas Abel, 11 May 2017
--
-- Guest lecture in class
--
--   Finite Automata Theory and Formal Languages
--   TMV027/DIT321, LP4 2017
--   http://www.cse.chalmers.se/edu/course/TMV027/

-- Input file to happy to generate parser for arithmetical expressions.

-- A happy file consists of 6 parts.
-- Haskell code is contained in { braces }.

-- 1. Haskell prologue: module header, import statements.

{
module CalcParser where

import CalcEval
import CalcLexer
}

-- 2. Basic declarations

-- The name of our parsing function.

%name calcParser

-- The haskell code to invoke for parse errors.
-- The function parseError is defined below.

%error { parseError }

-- 3. Tokens we use in grammar

%tokentype { Token }
%token
  '+'     { Plus   }
  '*'     { Times  }
  '('     { LParen }
  ')'     { RParen }
  '='     { Equals }
  'where' { Where  }
  num     { Number $$ }  -- $$ holds the value of the token
  id      { Ident  $$ }

-- 4. Magic separator

%%

-- 5. Productions of our grammar

-- The first non-terminal is what is parsed by "calcParser".
-- The attached haskell code computes the value of this production.
-- $n refers to the value of the nth item.

Expr
  : Sum                     { $1 }
  | Sum 'where' id '=' Expr { local $3 $5 $1 }

Sum
  : Product           { $1 }
  | Sum '+' Product   { plus $1 $3 }

Product
  : Atom              { $1 }
  | Product '*' Atom  { times $1 $3 }

Atom
  : num               { number   $1 }
  | id                { variable $1 }
  | '(' Expr ')'      { $2 }

-- 6. Haskell epilogue: any Haskell code.

{
parseError :: [Token] -> a
parseError toks = error ("Parse error at " ++ show (take 5 toks))
}
